! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      !---------------------------------------------------------------
      subroutine bdfi42r4r(output,no,start,step,xcen,ycen,npix,inst,ierr)
      !---------------------------------------------------------------

      use dal
      use global
      use cgenl

      implicit none

!      include 'include/cgenl_f90.cmn'

      integer     maxdim,naxis,ix,iy
      parameter   (maxdim=2)
      integer npix(maxdim)
      integer     no, ierr, atype
      character(len=1024), intent(out) :: inst
      real(kind=single) :: r4huge
      real(kind=double) :: r8huge
      real(kind=double) :: dist1, dist2 
      real(kind=double) :: cos_pa, sin_pa
      real(kind=double) :: rr, dr,r0r, d0r, dra, ddec
      real(kind=single) :: xr, yr
      integer :: xcen, ycen
!      real, dimension(nelem1,nelem2) :: r4arr
!      real        exposure 
      double precision start(maxdim),step(maxdim)
      character*(*) output

      type(datasett) :: set
      type(arrayt) :: arr
      integer, dimension(:), pointer :: shape

      integer(kind=int32), dimension(:,:), pointer :: i4arr
      integer(kind=int16), dimension(:,:), pointer :: i2arr
      integer(kind=int8), dimension(:,:), pointer :: i1arr
      real(kind=single), dimension(:,:), pointer :: r4arr
      real(kind=double), dimension(:,:), pointer :: r8arr

      r4huge=huge(1)
      r8huge=huge(1)

      set = dataset (output, read)
!      exposure = real32Attribute(set,"EXPOSURE")
      arr = array(set, "PRIMARY", read)
      shape => dimensions(arr)      
      npix(1) = shape(0)
      npix(2) = shape(1) 

      ipa_x = npix(1)
      ipa_y = npix(2)
      idm_x = ipa_x
      idm_y = ipa_y
      allocate (ifeld(ipa_x,ipa_y))
      ifeld = 0.

    atype = arrayDataType(arr)
    if (atype == Integer8) then
      i1arr => int8array2data(arr)
      do iy = 1,npix(2)
        do ix = 1,npix(1)
          ifeld(ix,iy) = i1arr(ix-1,iy-1)
        end do
      end do 
    else if (atype == Integer16) then
      i2arr => int16array2data(arr)
      do iy = 1,npix(2)
        do ix = 1,npix(1)
          ifeld(ix,iy) = i2arr(ix-1,iy-1)
        end do
      end do 
    else if (atype == Integer32) then 
      i4arr => int32array2data(arr)
      do iy = 1,npix(2)
        do ix = 1,npix(1)
          ifeld(ix,iy) = i4arr(ix-1,iy-1)
        end do
      end do 
    else if (atype == Real32) then
      r4arr => Real32array2data(arr)
      do iy = 1,npix(2)
        do ix = 1,npix(1)
          ifeld(ix,iy) = nint(min(r4arr(ix-1,iy-1),r4huge))
        end do
      end do 
    else if (atype == Real64) then
      r8arr => Real64array2data(arr)
      do iy = 1,npix(2)
        do ix = 1,npix(1)
          ifeld(ix,iy) = nint(min(r8arr(ix-1,iy-1),r8huge))
        end do
      end do 
    else
      call error("WrongType","Input image has wrong type")
    end if

    start(1) = 1d0
    start(2) = 1d0
    step(1) = 1d0
    step(2) = 1d0

    if (hasAttribute(set,"INSTRUME")) then 
      inst = stringAttribute(set,"INSTRUME")
    else if (hasAttribute(arr,"INSTRUME")) then
      inst = stringAttribute(arr,"INSTRUME")
    end if 

    if (hasAttribute(set,"RA_PNT")) then
      ra_pnt = real64Attribute(set,"RA_PNT")
    else  
      if (hasAttribute(arr,"RA_PNT")) then
        ra_pnt = real64Attribute(arr,"RA_PNT")
      else
        ra_pnt = 0d0
        call warning("MissingAttribute","Keyword RA_PNT is missing")
      end if
    end if

    if (hasAttribute(set,"DEC_PNT")) then
      dec_pnt = real64Attribute(set,"DEC_PNT")
    else  
      if (hasAttribute(arr,"DEC_PNT")) then
        dec_pnt = real64Attribute(arr,"DEC_PNT")
      else
        dec_pnt = 0d0
        call warning("MissingAttribute","Keyword DEC_PNT is missing")
      end if
    end if

    if (hasAttribute(set,"PA_PNT")) then
      pa_pnt = real64Attribute(set,"PA_PNT")
    else  
      if (hasAttribute(arr,"PA_PNT")) then
        pa_pnt = real64Attribute(arr,"PA_PNT")
      else
        pa_pnt = 0d0
        call warning("MissingAttribute","Keyword PA_PNT is missing")
      end if
    end if

    if (hasAttribute(arr,"CRPIX1")) then
      crpix1 = real64Attribute(arr,"CRPIX1")
    else
      if (hasAttribute(set,"CRPIX1")) then
        crpix1 = real64Attribute(set,"CRPIX1")
      else
        crpix1 = 0d0
        call warning("MissingAttribute","Keyword CRPIX1 is missing")
      end if
    end if

    if (hasAttribute(arr,"CRPIX2")) then
      crpix2 = real64Attribute(arr,"CRPIX2")
    else
      if (hasAttribute(set,"CRPIX2")) then
        crpix2 = real64Attribute(set,"CRPIX2")
      else
        crpix2 = 0d0
        call warning("MissingAttribute","Keyword CRPIX2 is missing")
      end if 
    end if

    if (hasAttribute(arr,"CRVAL1")) then
      crval1 = real64Attribute(arr,"CRVAL1")
    else
      if (hasAttribute(set,"CRPIX1")) then
        crval1 = real64Attribute(set,"CRVAL1")
      else
        crval1 = 0d0
        call warning("MissingAttribute","Keyword CRVAL1 is missing")
      end if
    end if

    if (hasAttribute(arr,"CRPIX2")) then
      crval2 = real64Attribute(arr,"CRVAL2")
    else
      if (hasAttribute(set,"CRPIX2")) then
        crval2 = real64Attribute(set,"CRVAL2")
      else
        crval2 = 0d0
        call warning("MissingAttribute","Keyword CRVAL2 is missing")
      end if 
    end if

    if (hasAttribute(arr,"CDELT1")) then
      crdelt1 = real64Attribute(arr,"CDELT1")
    else
      if (hasAttribute(set,"CDELT1")) then
        crdelt1 = real64Attribute(set,"CDELT1")
      else
        crdelt1 = 0d0
        call warning("MissingAttribute","Keyword CRDELT1 is missing")
      end if
    end if

    if (hasAttribute(arr,"CDELT2")) then
      crdelt2 = real64Attribute(arr,"CDELT2")
    else
      if (hasAttribute(set,"CDELT2")) then
        crdelt2 = real64Attribute(set,"CDELT2")
      else
        crdelt2 = 0d0
        call warning("MissingAttribute","Keyword CDELT2 is missing")
      end if
    end if

    if (hasAttribute(arr,"CROTA2")) then
       crota2 = real64Attribute(arr,"CROTA2")
    else
      if (hasAttribute(set,"CROTA2")) then
        crota2 = real64Attribute(set,"CROTA2")
      else
        crota2 = 0d0
!        call warning("MissingAttribute","Keyword CROTA2 is missing")
      end if
    end if

    if (crdelt1 > 0.0 .and. crdelt2 > 0.0) then
      dist1=0.36/abs(crdelt1)
      dist2=0.36/abs(crdelt2)
    else
      dist1=320.0
      dist2=320.0
    end if
    if (crpix1 < -dist1 .or. crpix1 > npix(1)+dist2 .or. &
        crpix2 < -dist2 .or. crpix2 > npix(2)+dist2) then
      call error("WrongRefPix","Reference pixel is outside of FOV")
    end if

    cos_pa = dcos(crota2 * pi/180d0)
    sin_pa = dsin(crota2 * pi/180d0)
    rr = ra_pnt * pi/180d0
    dr = dec_pnt * pi/180d0  
    r0r = crval1 * pi/180d0
    d0r = crval2 * pi/180d0
    call spher2(rr, dr, r0r, d0r, dra, ddec)
    xr =   dra * cos_pa + ddec * sin_pa ! Vorzeichen ??
    yr = - dra * sin_pa + ddec * cos_pa ! 
    xcen = xr * 180d0/pi / crdelt1 + crpix1
    ycen = yr * 180d0/pi / crdelt2 + crpix2

    if (xcen < 1 .or. xcen > npix(1) .or. &
        ycen < 1 .or. ycen > npix(2)) then
      xcen=min(max(xcen,1),npix(1))
      ycen=min(max(ycen,1),npix(2))
      call warning("WrongPointDir","Pointing direction is outside of image")
    end if

    call release(arr)
    call release(set)

    !!! IT 2021/04 default values, if no subimage is determined from the mask
    ix0 = 1
    iy0 = 1
    ix1 = ipa_x
    iy1 = ipa_y
    iusex = ix1-ix0+1
    iusey = iy1-iy0+1

    end subroutine bdfi42r4r




