! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      !--------------------------------------------------------------------
      subroutine bdfi4r(output,no,start,step,npix,inst,ierr)
      !--------------------------------------------------------------------

      use dal
      use global
      use cgenl

      implicit none

!      include 'include/cgenl_f90.cmn'

      integer     maxdim,naxis,ix,iy
      parameter   (maxdim=2)
      integer npix(maxdim)
      character(len=1024), intent(in) :: inst
      integer     no, ierr, atype
!     real        exposure 
      double precision start(maxdim),step(maxdim)
      character(len=1024) :: ins1
      character*(*) output

      type(datasett) :: set
      type(arrayt) :: arr
      integer, dimension(:), pointer :: shape
      integer, dimension(:,:), pointer :: i4arr0
      real(kind=single), dimension(:,:), pointer :: r4arr
      
      real,dimension(ipa_x) :: isumx    !!! IT 2021/04 for subimages
      real,dimension(ipa_y) :: isumy

      set = dataset (output, read)
!      exposure = real32Attribute(set,"EXPOSURE")

! Horrible hack to get this routine to work with mask files where the
! main array is stored in the MASK extension - should be recoded - RDS
      if ( hasBlock( set, "MASK" ) ) then
          arr = array(set, "MASK", read)
      else
          arr = array(set, "PRIMARY", read)
      endif

      if (hasAttribute(set,"INSTRUME")) then 
        ins1 = stringAttribute(set,"INSTRUME")
      else if (hasAttribute(arr,"INSTRUME")) then
          ins1 = stringAttribute(arr,"INSTRUME")
      end if 
      if (trim(inst) /= trim(ins1)) &
        call error ("FileMismatch","Exposure map has wrong instrument") 

      shape => dimensions(arr)      
      npix(1) = shape(0)
      npix(2) = shape(1)
      ipa_x = npix(1)
      ipa_y = npix(2)
      idm_x = ipa_x
      idm_y = ipa_y
      allocate (mfeld(ipa_x,ipa_y))
      mfeld = 0
      atype = arrayDataType(arr)

      if (atype == Integer32) then
        i4arr0 => int32array2data(arr)
        do ix=1,npix(1)
          do iy=1,npix(2)
            mfeld(ix,iy) = i4arr0(ix-1,iy-1)
          end do
        end do
      end if
      if (atype == Real32) then
        r4arr => Real32array2data(arr)
        do ix=1,npix(1)
          do iy=1,npix(2)
            mfeld(ix,iy) = r4arr(ix-1,iy-1)
          end do
        end do
      end if
        
      start(1) = 1d0
      start(2) = 1d0
      step(1) = 1d0
      step(2) = 1d0

      call release(arr)
      call release(set)
      
      
      !! subimage of (1:ipa_x,1:ipa_y)                   !!! IT 2019/03
      !! code: cf emldetect, eboxdetect
      
      ! subimage coordinates - start values
      ix0=1
      iy0=1
      ix1=2
      iy1=2

      ! collapse
      isumx=sum(MFELD,2)
      isumy=sum(MFELD,1)

      ! determine coordinates of the subimage:
      ! - first column with a non-zero entry (i.e.: along x),  bottom-up
      do ix=1,ipa_x
         if (isumx(ix)>0) then
            ix0=ix
            exit
         end if
      end do
      ! - first row    with a non-zero entry (i.e.: along y),  bottom-up
      do iy=1,ipa_y
         if (isumy(iy)>0) then
            iy0=iy
            exit
         end if
      end do
      ! - last  column with a non-zero entry (i.e.: along x),  top-down
      do ix=ipa_x,1,-1
         if (isumx(ix)>0) then
            ix1=ix
            exit
         end if
      end do
      ! - last  row    with a non-zero entry (i.e.: along y),  top-down
      do iy=ipa_y,1,-1
         if (isumy(iy)>0) then
            iy1=iy
            exit
         end if
      end do

      ! add some margin and round for fft
      ix0 = max( (ix0-16)/16*16, 1 )
      iy0 = max( (iy0-16)/16*16, 1 )
      ix1 = min( (ix1+16)/16*16-1, ipa_x )
      iy1 = min( (iy1+16)/16*16-1, ipa_y )
      iusex = ix1-ix0+1
      iusey = iy1-iy0+1

      end subroutine bdfi4r
