C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      SUBROUTINE BKGIMG(NSX,NSY,SCF_MEX,FMIN,IERR)

C produces binned background-image without bright sources
C***********************************************************************
C
C   call_var.          type I/O description 
CP  IERR                I4    O error code
CP                              = 0 no error
CP  NSX                 I4  I   X-sampling for BKG-bins
CP  NSY                 I4  I   Y-sampling for BKG-bins
CP  SCALE               R4  I   background scale factor (usually 100.)
CP  IDM_X               I4  I   length of array in x-direction
CP  IDM_Y               I4  I   length of array in y-direction
CP  IEXMPX              I4  I   exposure image array
CP  SCF_MEX             R8  I   scale factor exposure image
CP  IDME_X              I4  i   length of exposure array in x-direction
CP  IDME_Y              I4  i   length of exposure array in y-direction
C
C***********************************************************************

      use global
      use nag_global

      IMPLICIT NONE
      INCLUDE 'include/cgenl.cmn'
!      INCLUDE 'include/cdbnag.cmn'

      CHARACTER*8   RTNAME
      INTEGER     I,IERR,II,J,M,N,NMAP,NSX,NSY,NX,NY,
     &              nsums,isums,isumm
      REAL        SCALE,FMIN,fnags(mtot)
      DOUBLE PRECISION SWNAG,VFNAG,VWNAG,SCF_MEX,NSUM,SEXP,SEXP2
      DOUBLE PRECISION fmax,etemp
      INTEGER     NNEAR,NCHECK,ICHECK(4),interflag(mtot)
      DOUBLE PRECISION SUMNEAR   


      SAVE

      IERR  = 0
      II = 0
      nsums = 0
      FMIN=1e30

      IF (MOD(IDM_X,NSX).NE.0) THEN
        NX = IDM_X/NSX + 1
      ELSE
        NX = IDM_X/NSX
      ENDIF
      IF (MOD(IDM_Y,NSY).NE.0) THEN
        NY = IDM_Y/NSY + 1
      ELSE
        NY = IDM_Y/NSY
      ENDIF


c      open(1,file='nsum.dat',status='unknown')
      DO N = 1,NY

C accumulate a strip with width NSX located at the position N
        DO M = 1,NX

          SEXP = 0.
          NSUM = 0.D0
          NMAP = 0

          DO J = (N-1)*NSY+1,min(ipa_y,N*NSY)
            DO  I = (M-1)*NSX+1,min(ipa_x,M*NSX)

C accumulate only if mask is .true.
              IF(MFELD(I,J).EQ.1)THEN

                NSUM = NSUM + ICHES(I,J)
                SEXP =  SEXP + DBLE(IEXMP(I,J))
                NMAP = NMAP+1
              ENDIF
            ENDDO
          ENDDO


          SEXP = SEXP * SCF_MEX

C one background subcell ready / does it contain enough counts
          II = II+1 
          XNAG(II) = (DBLE(M)-0.5D0)*DBLE(NSX)
          YNAG(II) = (DBLE(N)-0.5D0)*DBLE(NSY)

c          write(1,*) xnag(ii),ynag(ii),nsum,nmap,sexp

          IF (NMAP.GT.1) THEN 

C set negative bins to zero:

          NSUM=MAX(NSUM,0.D0)

C determine spline parameters
            IF (SEXP.GT.1.) THEN
               FNAG(II) = (NSUM/SEXP)
C              Reduce dynamic range of the input data, removed 17/10/03
              FNAG(II) = (NSUM/SEXP)**0.1
            ELSE
              FNAG(II) = -999D0
            ENDIF

            ENAG(II) = SEXP/SQRT(MAX(NSUM,1.D0))
            WNAG(II) = 1.0
            if (fnag(ii) > 0) then
              nsums = nsums+1
              if (nsums == 1) then
                fnags(1) = fnag(ii)
              else
                do isums=1,nsums
                  if (fnags(isums)>fnag(ii)) exit
                end do
                isums=min(nsums,isums)
                do isumm=nsums,isums+1,-1
                  fnags(isumm)=fnags(isumm-1)
                end do              
                fnags(isums)=fnag(ii)
              end if
            end if  
          ELSE

C   flag spline parameters for empty bins
            FNAG(II) = -999D0
            WNAG(II) = -999D0
            ENAG(II) = 0D0
          ENDIF
          

          VFNAG = FNAG(II)
          VWNAG = WNAG(II)          

C write background content to preliminary smoothed image
          DO J = (N-1)*NSY+1,min(ipa_y,N*NSY)
            DO I = (M-1)*NSX+1,min(ipa_x,M*NSX)
              IFELD(I,J) = VFNAG
              JFELD(I,J) = VWNAG
            ENDDO
          ENDDO
        ENDDO
      ENDDO
      NPTS = II
 
c      close(1)
C put mean value on pixel that had too few counts
C they have been set to -999.
C if empty bin has >=2 near valid neighbours, replace with interpolation
     
      fmax=maxval(enag)
      fmin=fnags(max(nsums/5,1))
      DO II = 1,NPTS
        if (fnag(ii) .le. 0d0) then
           fnag(ii) = fmin
c          check nearest bins  
           if (mod(ii,nx) > 1) then
             if ((ii / nx) < ny-1 .and. (ii / nx) > 0) then
                ncheck=4
                icheck=(/ii-1,ii+1,ii-nx,ii+nx/)
             else if ((ii / nx) == 0) then
                ncheck=3
                icheck=(/ii-1,ii+1,ii+nx,0/) 
             else if ((ii / nx) == ny-1) then             
                ncheck=3
                icheck=(/ii-1,ii+1,ii-nx,0/) 
             endif
           else if (mod(ii,nx) == 1) then
             if ((ii / nx) < ny-1 .and. (ii / nx) > 0) then
                ncheck=3
                icheck=(/ii+1,ii-nx,ii+nx,0/)
             else if ((ii / nx) == 0) then
                ncheck=2
                icheck=(/ii+1,ii+nx,0,0/)
             else if ((ii / nx) == ny-1) then             
                ncheck=2 
                icheck=(/ii+1,ii-nx,0,0/) 
             endif
           else if (mod(ii,nx) == 0) then
             if ((ii / nx) < ny-1 .and. ((ii-1) / nx) > 0) then
                ncheck=3
                icheck=(/ii-1,ii-nx,ii+nx,0/)
             else if ((ii / nx) == 1) then
                ncheck=2
                icheck=(/ii-1,ii+nx,0,0/) 
             else if ((ii / nx) == ny-1) then             
                ncheck=2
                icheck=(/ii-1,ii-nx,0,0/) 
             endif
           endif
c
           interflag(ii)=0
           nnear = 0
           sumnear=0.D0
           do i=1,ncheck
             if ((fnag(icheck(i))>0.D0)
     &            .and. (interflag(icheck(i)) == 0)
     &             .and. ( enag(ii)/fmax > 0.15) ) then
               nnear=nnear+1
               sumnear=sumnear+fnag(icheck(i))
             endif
           end do             
           if (nnear > 1) then
c comment out to not replace fnag
             fnag(ii)=sumnear/float(nnear)
             interflag(ii)=1
           endif

           wnag(ii) = wmin
           enag(ii) = 0d0
        end if
c        if (enag(ii) .le. 15.) then        
c           wnag(ii) = 0.2
c        endif
        etemp=enag(ii)
        wnag(ii) = max(wmin,(etemp/fmax))
      ENDDO



c write binned data to ascii file:

c      write(*,*) "create binned data file"
 
c      open(1,file='binned.dat',status='unknown')      

c      DO II = 1,NPTS  
c        write(1,*) xnag(ii),ynag(ii),fnag(ii),wnag(ii),enag(ii),
c     &             interflag(ii) 

c      ENDDO
c      close(1)

      RETURN
      END


