C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      SUBROUTINE EXDB

CC  Pointing Background map program with bicubic splines
C
C************************ FFORM VERSION 1.1 ************  7-NOV-89 15:12
C
CA  author : GRH        date: 1-DEC 1986 20:21
CU  update : CRI        date: 18-MAR-1988 11:21
CU  update : PTK        date: 24-AUG-1992 14:28 EXDBOUT
CU  update : PTK        date: 01-DEC-1992 16:12 IEXMP,SCF_MEX,
CU                                              IDME_X,IDME_Y
CU  update : PTK        date: 14-DEC-1992 22:06 dbini
C
CT  status: not tested
C
C   general description 
CG    P O I N T I N G   B A C K G R O U N D - M A P  P R O G R A M M
CG    with bicubic splines uses an image and the
CG    appropriate LDETEC source list plus an exposure map. bright
CG    sources according to DB_SNRM are removed from the exposure map,
CG    then a binned background image without bright sources is
CG    accumulated and smoothed by bicubic splines.
C
C   call_var.          type I/O description 
CP 
C
C   include_block name          description 
CI  RCOMMON:CDBIO.CMN           DBPAR parameters common block
CI  RCOMMON:CGENL.CMN           general parameter common block
CI  RCOMMON:CSOIO.CMN           SOPAR parameter common block
CI  RCOMMON:CDBIO.INC           DBPAR parameters common block, DATA
CI  RCOMMON:CGENL.INC           general parameter common block, DATA
CI  RCOMMON:CSOIO.INC           SOPAR parameter common block, DATA
C
C   routines_called    type     description 
CR  BKGIMG              SR      produces binned background-image
CR  BLIND               SR      blinds exposure map at detected sources
CR  CHEESE              SR      folds image with blinded exposure map
CR  CHIOUT              SR      removes 4-sigma excesses
CR  EXDBIN              SR      read input files
CR  EDBOUT              SR      write results
CR  HFLAG               R4      output flag handling routine
CR  RESTOR              SR      reconstructs background map
CR  SPLIN2              SR      calculates spline parameters
CR  WRFLAG              SR      writes to FLAG output stream
CR  WRERR               SR      write error message
CR  WRLOG               SR      write to log file
CR  STSEPI              SR      exit from MIDAS
CR  STSPRO              SR      go into MIDAS
CR  TBLDBI              SR      read table file for task EXDB
CR  TBLSOI              SR      read table file for task SO
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C   NOIM        number identifier of IMAGE
C   NOBL        number identifier of BLDFL
C   NOBI        number identifier of BINFL
C   NOBA        number identifier of BACMP
C   NOWB        number identifier of WBNFL
C   TILS        number identifier of LSLST

      use global
      use nag_global
!      use ootmap_mod
      use nelder_mead_test_mod
      use nelder_mead
      use esp_smooth_mod           !!! IT 2016/01


      IMPLICIT NONE

      INTEGER     NTOT
      PARAMETER     (NTOT=10000)
      INTEGER     IECH, IERR, ISTAT, KMERK,
     &              NOUT, NSTEP, NSX, NSY,
     &              NOIM,NOMF,TILS,MAX_MEX   !!! , LOU, K, IWIND
              
      INTEGER     P   !!! , READEXT, I
      REAL        FMIN, X(NTOT), Y(NTOT)  !!! , ABSTM, ABSTS, BKG, SRC
      REAL        ISNR(NTOT),SCTS(NTOT)
      DOUBLE PRECISION
     &        FMAX,SCF_MEX,MAX_IMG    !!!,SFNAG,SWNAG
!!!      CHARACTER*1   DB_CMAP
!!!      CHARACTER*2   IO
!!!      CHARACTER*8   RTNAME
!!!      CHARACTER*12  MIS_ID,DET_ID
!!!      CHARACTER*70  PAR
!!!      CHARACTER*72  TEXT
      CHARACTER*92  messtxt  !!!, CNAME
      LOGICAL     LOG,FIRST  !!! ,WRITE

      INCLUDE 'include/cgenl.cmn'
      INCLUDE 'include/csoio.cmn'
      INCLUDE 'include/cdbio.cmn'
c      INCLUDE 'include/cdbnag.cmn'
      INCLUDE 'include/cimsky.cmn'

!!!      DATA          RTNAME   /'EXDB'/
!!!      DATA          WRITE    /.FALSE./
      DATA          LOG      /.FALSE./

      INCLUDE 'include/cgenl.inc'
      INCLUDE 'include/csoio.inc'
      INCLUDE 'include/cdbio.inc'
      INCLUDE 'include/cdbnag.inc'

       IERR=0

      CALL DBPARI 


      IECH = 1
 
      CALL EXDBIN(NTOT,IECH,KMERK,X,Y,ISNR,SCTS,NOIM,NOMF,TILS,
     &            SCF_MEX,MAX_MEX,MAX_IMG,ISTAT)

 
C model OOT events for pn, if required
      if (withoot) then
        if (idinst .eq. 1) then
          call ootcreate
          call ootsubtract
        else 
          write (messtxt,*) "OOT background only calculated for EPN" 
          call wrmess (messtxt)
        endif
      endif


C blind sources in the mask
      CALL BLIND(NTOT,KMERK,X,Y,ISNR,SCTS,DB_SNRM,ISTAT)


C create swiss cheese image
      CALL CHEESE(ISTAT)
 
C accumulate binned background image
           ! nsx=20
           ! nsy=20 
           NPX = db_nspline ! 2**DB_NSPLINE
           NPY = db_nspline ! 2**DB_NSPLINE
!           NPX=64
!           NPY=64
           nsx=idm_x/npx+1
           nsy=idm_y/npy+1 
           NADRES=(NPX-7)*(NPY-7)
           NC=(NPX-4)*(NPY-4)
           P = 4 + 3*(NPY-4)
           NWORK=2*NC*(2+P)+P

           ISTAT = 1 


ccc IT 2016/01 - new method smooth
c               IF (method .eq. 'spline') THEN
c                 CALL BKGIMG(NSX,NSY,SCF_MEX,FMIN,ISTAT)
c               ELSE IF  (method .eq. 'model') THEN
c                 CALL  BINEXP(NSX,NSY)
c               ENDIF
           IF  (method .eq. 'model') THEN
              CALL BINEXP(NSX,NSY)
           ELSE 
              CALL BKGIMG(NSX,NSY,SCF_MEX,FMIN,ISTAT)
           ENDIF


              NOUT=1
              NSTEP=0
              FIRST = .TRUE.
              LOG=.FALSE.
              !LOG = .TRUE.
C              CALL STTPUT('          ITERATION STEPS STARTED',ISTAT)
C              CALL STTPUT('           STEP  CHISQR  NOUTS',ISTAT)



ccc IT 2016/01 - new method smooth
           IF  (method .eq. 'smooth') THEN
              CALL  ESP_SMOOTH
           ELSE 


c            WNAG_SAVE(:)=WNAG(:)

C           do iterations removing count excesses
            DO WHILE(NSTEP.LT.DB_NSTP.AND.NOUT.GT.0)

               NSTEP=NSTEP+1
c               WNAG(:)=WNAG_SAVE(:)
               IF (method .eq. 'spline') THEN
C              do spline fit
                 CALL SPLIN2(FMAX,LOG,FIRST,ISTAT)
               ELSE IF  (method .eq. 'model') THEN
                 CALL  nelder_mead_test
               ENDIF
C              throw out > N sigma excesses
               CALL CHIOUT(DB_CHIS(IECH),NOUT,ISTAT)    
               DB_OUTS(IECH) = DB_OUTS(IECH)+NOUT
               if (nstep == 1) then
                 write (messtxt,*) "removing",int(db_esig), 
     &             "sigma excesses:"
                 call wrmess (messtxt)
               end if
               write(messtxt,*) 
     &           "red. chisqr:",db_chis(iech)," dof:",ndof,
     &           " removed bins:",db_outs(iech) 
               call wrmess(messtxt)
            END DO


         ENDIF  ! if method smooth



C   reconstruction of background map 
            CALL RESTOR(FMAX,BSCAL,DB_NRES,LOG,SCF_MEX,
     &                  DB_EXP,FMIN,ISTAT)

         




       if (withoot .and. idinst .eq. 1) then
         call ootadd
       endif

C   write output images
       CALL EDBOUT(IECH,ISTAT)


      END



