! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: nelder_mead_test_mod
!
! Authors:  V.hamabryan - AIP
!          
! Description:  A PROGRAM FOR FUNCTION MINIMIZATION USING THE SIMPLEX METHOD
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module nelder_mead_test_mod

!  use dal
!  use param  
!  use errorHandling
  use nelder_mead

  use global
  use nag_global

  implicit none 
INTEGER, PARAMETER :: dp = SELECTED_REAL_KIND(14, 60)
REAL (dp)          :: object, p(2), simp, step(2), stopcr, var(2)
INTEGER            :: ier, iprint, iquad, maxf, nloop, nop
LOGICAL            :: first
  
  private 

  public nelder_mead_test

contains 
  
  !---------------------------------------------------------------------
  subroutine nelder_mead_test 
  !---------------------------------------------------------------------
! Set up starting values & step sizes.
! Parameters p(1), p(2), p(3) are x, y & z.

p(1) = 1.d-8
p(2) = 1.d-8

step = 1.d-9
nop = 2

! Set max. no. of function evaluations = 250, print every 10.

maxf = 250
iprint = -1

! Set value for stopping criterion.   Stopping occurs when the
! standard deviation of the values of the objective function at
! the points of the current simplex < stopcr.

stopcr = 1.d-7
nloop = 4

! Fit a quadratic surface to be sure a minimum has been found.

iquad = 1

! As function value is being evaluated in REAL (dp), it
! should be accurate to about 15 decimals.   If we set simp = 1.d-6,
! we should get about 9 dec. digits accuracy in fitting the surface.

simp = 1.d-6

! Now call MINIM to do the work.

first = .true.
DO
  CALL minim(p, step, nop, object, maxf, iprint, stopcr, nloop,   &
             iquad, simp, var, objfun, ier)

! If ier > 0, try a few more function evaluations.

  IF (ier == 0) EXIT
  IF (.NOT. first) then
    write(*,*) "error in minim:",  ier
    STOP
  ENDIF
  first = .false.
  maxf = 100
END DO

! Successful termination.

!WRITE(*, 900) object, p
900 FORMAT(' Success !'/' Objective function = ', f12.6/ ' at: ', 3f12.6)
!WRITE(*, 910) var
910 FORMAT(' Elements of var = ', 3f12.6)

backparam(1)=p(1)
backparam(2)=p(2)

    

  end subroutine nelder_mead_test

 SUBROUTINE objfun(p, func)

! This is the subroutine which the user must write.
! Remember that we are minimizing the negative of the function we
! really want to maximize.

IMPLICIT NONE
INTEGER, PARAMETER     :: dp = SELECTED_REAL_KIND(14, 60)
REAL (dp), INTENT(IN)  :: p(:)
REAL (dp), INTENT(OUT) :: func
!     Local variables

!integer, parameter :: m = 129818  !!! band 1
!INTEGER,  parameter :: m = 2202    !!! band 1
!integer, parameter :: m = 119734  !!! band 2
!INTEGER,  parameter :: m = 2095    !!! band 2
!integer, parameter :: m = 122762  !!! band 3
!INTEGER,  parameter :: m = 2117    !!! band 3
!integer, parameter :: m = 125371  !!! band 4
!INTEGER,  parameter :: m = 2145    !!! band 4
!Integer, parameter :: m = 130540  !!! band 5
!INTEGER,  parameter :: m = 2208    !!! band 5

!REAL (dp)            :: tau1(m), tau2(m),fobs(m)
REAL (dp)            :: fct,fmod,lnprob
REAL (dp)            :: x, y
INTEGER :: i

!  open(1,file='tau1_ori1000',status='old')
!  read(1,*) tau1(1:m)
!  close(1)
!  open(1,file='tau2_ori1000',status='old')
!  read(1,*) tau2(1:m)
!  close(1)
!  open(1,file='obs_ori1000',status='old')
!  read(1,*) fobs(1:m)
!  close(1)
!  open(1,file='tau15000',status='old')
!  read(1,*) tau1(1:m)
!  close(1)
!  open(1,file='tau25000',status='old')
!  read(1,*) tau2(1:m)
!  close(1)
!  open(1,file='obs5000',status='old')
!  read(1,*) fobs(1:m)
!  close(1)

x = p(1)
y = p(2)
fct=1.d0
func=0.0d0

! exp2binned: binned, non-vignetted expusure map
! expbinned: binned, vignetted expusure map
! imbinned: binned cheesed input image

Do i=1,npts
   if (wnag(i) .gt. 0.D0) then
     ffnag(i)=x*exp2binned(i)/fct+y*expbinned(i)/fct
     func=func-2.d0*(fnag(i)*DLOG(ffnag(i))-ffnag(i))

   endif
!   lnprob=lnprob+fmod+alngam(fobs(i)+0.5d0)-(fobs(i)-0.5d0)*DLOG(fmod)
!write(*,*) x,y,func,fmod
enddo
!func=lnprob
RETURN
END SUBROUTINE objfun

FUNCTION alngam(xvalue) RESULT(fn_val)

!     ALGORITHM AS245  APPL. STATIST. (1989) VOL. 38, NO. 2

!     Calculation of the logarithm of the gamma function

! ELF90-compatible version by Alan Miller
! Latest revision - 29 November 1997

! N.B. Argument IFAULT has been removed

IMPLICIT NONE
INTEGER, PARAMETER    :: dp = SELECTED_REAL_KIND(15, 100)
REAL (dp), INTENT(IN) :: xvalue
REAL (dp)             :: fn_val

! Local variables
REAL (dp) :: x, x1, x2, y

!     Coefficients of rational functions

REAL (dp), PARAMETER :: r1(9) = (/ -2.66685511495_dp, -24.4387534237_dp,  &
                                   -21.9698958928_dp,  11.1667541262_dp,  &
                                    3.13060547623_dp,  0.607771387771_dp, &
                                    11.9400905721_dp,  31.4690115749_dp,  &
                                    15.2346874070_dp /)
REAL (dp), PARAMETER :: r2(9) = (/ -78.3359299449_dp, -142.046296688_dp,  &
                                    137.519416416_dp,  78.6994924154_dp,  &
                                    4.16438922228_dp,  47.0668766060_dp,  &
                                    313.399215894_dp,  263.505074721_dp,  &
                                    43.3400022514_dp /)
REAL (dp), PARAMETER :: r3(9) = (/ -2.12159572323E5_dp,  2.30661510616E5_dp,  &
                                    2.74647644705E4_dp, -4.02621119975E4_dp,  &
                                   -2.29660729780E3_dp, -1.16328495004E5_dp,  &
                                   -1.46025937511E5_dp, -2.42357409629E4_dp,  &
                                   -5.70691009324E2_dp /)
REAL (dp), PARAMETER :: r4(5) = (/ 0.279195317918525_dp, 0.4917317610505968_dp, &
                                   0.0692910599291889_dp, 3.350343815022304_dp, &
                                   6.012459259764103_dp /)

!     Fixed constants

REAL (dp), PARAMETER :: alr2pi = 0.918938533204673_dp, four = 4._dp,  &
                        half = 0.5_dp, one = 1._dp, onep5 = 1.5_dp,   &
                        twelve = 12._dp, zero = 0._dp

!     Machine-dependant constants.
!     A table of values is given at the top of page 399 of the paper.
!     These values are for the IEEE double-precision format for which
!     B = 2, t = 53 and U = 1023 in the notation of the paper.

REAL (dp), PARAMETER :: xlge = 5.10E6_dp, xlgst = HUGE(1.0_dp)

x = xvalue
fn_val = zero

!     Test for valid function argument

IF (x >= xlgst) THEN
  WRITE(*, *) 'AS 245: Argument x too large'
  RETURN
END IF
IF (x <= zero) THEN
  WRITE(*, *) 'AS 245: Argument x <= 0'
  RETURN
END IF

!     Calculation for 0 < X < 0.5 and 0.5 <= X < 1.5 combined

IF (x < onep5) THEN
  IF (x < half) THEN
    fn_val = -LOG(x)
    y = x + one
    
!     Test whether X < machine epsilon
    
    IF (y == one) RETURN
  ELSE
    fn_val = zero
    y = x
    x = (x - half) - half
  END IF
  fn_val = fn_val + x * ((((r1(5)*y + r1(4))*y + r1(3))*y + r1(2))*y + r1(1)) / &
                    ((((y + r1(9))*y + r1(8))*y+ r1(7))*y + r1(6))
  RETURN
END IF

!     Calculation for 1.5 <= X < 4.0

IF (x < four) THEN
  y = (x - one) - one
  fn_val = y * ((((r2(5)*x + r2(4))*x + r2(3))*x + r2(2))*x + r2(1)) /  &
               ((((x + r2(9))*x + r2(8))*x + r2(7))*x+ r2(6))
  RETURN
END IF

!     Calculation for 4.0 <= X < 12.0

IF (x < twelve) THEN
  fn_val = ((((r3(5)*x + r3(4))*x + r3(3))*x + r3(2))*x + r3(1)) /  &
           ((((x + r3(9))*x + r3(8))*x + r3(7))*x + r3(6))
  RETURN
END IF

!     Calculation for X >= 12.0

y = LOG(x)
fn_val = x * (y - one) - half * y + alr2pi
IF (x > xlge) RETURN
x1 = one / x
x2 = x1 * x1
fn_val = fn_val + x1 * ((r4(3)*x2 + r4(2))*x2 + r4(1)) /  &
         ((x2 + r4(5))*x2 + r4(4))
RETURN
END FUNCTION alngam

end module nelder_mead_test_mod
